package com.example;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.Service;
import android.content.Intent;
import android.os.Build;
import android.os.Handler;
import android.os.IBinder;
import android.os.Looper;
import android.util.Log;
import androidx.core.app.NotificationCompat;
import androidx.core.app.NotificationManagerCompat;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

public class MyBackgroundService extends Service {
    private static final String TAG = "MyBackgroundService";
    private static final String CHANNEL_ID = "BackgroundServiceChannel";
    private static final int NOTIFICATION_ID = 1;
    private static final int FOREGROUND_ID = 1001;
    
    private Handler handler;
    private Runnable runnable;
    private int interval = 60000; // Standard 1 Minute
    private String serverUrl = "https://wavi.at/check.php";
    private boolean isServiceRunning = false;

    @Override
    public void onCreate() {
        super.onCreate();
        createNotificationChannel();
        handler = new Handler(Looper.getMainLooper());
        startForeground();
    }

    private void createNotificationChannel() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            NotificationChannel serviceChannel = new NotificationChannel(
                CHANNEL_ID,
                "Background Service Channel",
                NotificationManager.IMPORTANCE_DEFAULT
            );
            NotificationManager manager = getSystemService(NotificationManager.class);
            if (manager != null) {
                manager.createNotificationChannel(serviceChannel);
            }
        }
    }

    private void startForeground() {
        NotificationCompat.Builder builder = new NotificationCompat.Builder(this, CHANNEL_ID)
                .setContentTitle("Service läuft")
                .setContentText("Überwache Server-Status")
                .setSmallIcon(android.R.drawable.ic_dialog_info)
                .setPriority(NotificationCompat.PRIORITY_LOW);

        startForeground(FOREGROUND_ID, builder.build());
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        if (!isServiceRunning) {
            isServiceRunning = true;
            
            if (intent != null && intent.getExtras() != null) {
                String newUrl = intent.getStringExtra("serverUrl");
                if (newUrl != null && !newUrl.isEmpty()) {
                    serverUrl = newUrl;
                }
                interval = intent.getIntExtra("interval", 60000);
            }

            runnable = new Runnable() {
                @Override
                public void run() {
                    checkServer();
                    if (isServiceRunning) {
                        handler.postDelayed(this, interval);
                    }
                }
            };
            
            handler.post(runnable);
        }
        
        return START_STICKY;
    }

    private void checkServer() {
        new Thread(() -> {
            HttpURLConnection connection = null;
            BufferedReader reader = null;
            
            try {
                URL url = new URL(serverUrl);
                connection = (HttpURLConnection) url.openConnection();
                connection.setRequestMethod("GET");
                connection.setConnectTimeout(5000);
                connection.setReadTimeout(5000);
                
                reader = new BufferedReader(new InputStreamReader(connection.getInputStream()));
                String response = reader.readLine();
                
                if (response != null && !"0".equals(response.trim())) {
                    sendNotification(response);
                }
                
            } catch (Exception e) {
                Log.e(TAG, "Fehler beim Abrufen der URL: " + e.getMessage());
            } finally {
                try {
                    if (reader != null) reader.close();
                    if (connection != null) connection.disconnect();
                } catch (Exception e) {
                    Log.e(TAG, "Fehler beim Schließen der Verbindung", e);
                }
            }
        }).start();
    }

    private void sendNotification(String message) {
        try {
            NotificationCompat.Builder builder = new NotificationCompat.Builder(this, CHANNEL_ID)
                    .setSmallIcon(android.R.drawable.ic_dialog_alert)
                    .setContentTitle("Server Antwort")
                    .setContentText(message)
                    .setPriority(NotificationCompat.PRIORITY_HIGH)
                    .setAutoCancel(true);

            NotificationManagerCompat manager = NotificationManagerCompat.from(this);
            
            // POST_NOTIFICATIONS permission check (Android 13+)
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                if (checkSelfPermission(android.Manifest.permission.POST_NOTIFICATIONS) 
                        == android.content.pm.PackageManager.PERMISSION_GRANTED) {
                    manager.notify(NOTIFICATION_ID, builder.build());
                }
            } else {
                manager.notify(NOTIFICATION_ID, builder.build());
            }
        } catch (Exception e) {
            Log.e(TAG, "Fehler beim Senden der Notification", e);
        }
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public void onDestroy() {
        isServiceRunning = false;
        if (handler != null && runnable != null) {
            handler.removeCallbacks(runnable);
        }
        super.onDestroy();
    }
}